'use strict';

var _taggedTemplateLiteral = require('babel-runtime/helpers/tagged-template-literal')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _browserSync = require('browser-sync');

var _browserSync2 = _interopRequireDefault(_browserSync);

var _co = require('co');

var _co2 = _interopRequireDefault(_co);

var _coPrompt = require('co-prompt');

var _coPrompt2 = _interopRequireDefault(_coPrompt);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _ncp = require('ncp');

var _open = require('open');

var _open2 = _interopRequireDefault(_open);

var _npm = require('npm');

var _dedent = require('dedent');

var _dedent2 = _interopRequireDefault(_dedent);

var _config = require('../config');

var _helpers = require('../helpers');

var _core = require('../core');

(0, _bluebird.promisifyAll)(_fs2['default']);
var npmLoadAsync = (0, _bluebird.promisify)(_npm.load);
var mkdirpAsync = (0, _bluebird.promisify)(_mkdirp2['default']);
var ncpAsync = (0, _bluebird.promisify)(_ncp.ncp);
var clc = (0, _helpers.cliColor)();

exports.init = init;
exports.config = config;
exports.newFile = newFile;
exports.run = run;
exports.openFile = openFile;

// Open a file using browser, text-editor
function openFile(type, sitePath, file) {
    if (type === 'file') {
        (0, _open2['default'])(_path2['default'].resolve(sitePath, file));
    } else {
        // TODO unless we add a --no-watch flag, we can just delegate this to browser-sync
        (0, _open2['default'])(file);
    }
}

function config(passedPath) {
    var _skipFindRoot = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

    var sitePath = _skipFindRoot ? passedPath : (0, _helpers.findHarmonicRoot)(passedPath);

    if (!sitePath) {
        (0, _helpers.displayNonInitializedFolderErrorMessage)();
        throw new _helpers.MissingFileError();
    }

    var manifest = _path2['default'].join(sitePath, 'harmonic.json');

    return new _Promise(function (fulfill, reject) {
        (0, _co2['default'])(_regeneratorRuntime.mark(function callee$2$0() {
            var templateObj, _p, config;

            return _regeneratorRuntime.wrap(function callee$2$0$(context$3$0) {
                while (1) switch (context$3$0.prev = context$3$0.next) {
                    case 0:
                        _p = function _p(message) {
                            return (0, _coPrompt2['default'])(clc.message(message));
                        };

                        console.log(clc.message('This guide will help you to create your Harmonic configuration file\n' + 'Just hit enter if you are ok with the default values.\n\n'));

                        templateObj = {
                            name: 'Awesome website',
                            title: 'My awesome static website',
                            domain: 'http://awesome.com',
                            subtitle: 'Powered by Harmonic',
                            author: 'Jaydson',
                            description: 'This is the description',
                            bio: 'Thats me',
                            theme: 'harmonic-theme-default',
                            preprocessor: 'stylus',
                            posts_permalink: ':language/:year/:month/:title',
                            pages_permalink: ':language/pages/:title',
                            header_tokens: ['<!--', '-->'],
                            index_posts: 10,
                            i18n: {
                                'default': 'en',
                                languages: ['en', 'pt-br']
                            }
                        };
                        context$3$0.next = 5;
                        return _p('Site name: (' + templateObj.name + ') ');

                    case 5:
                        context$3$0.t0 = context$3$0.sent;

                        if (context$3$0.t0) {
                            context$3$0.next = 8;
                            break;
                        }

                        context$3$0.t0 = templateObj.name;

                    case 8:
                        context$3$0.t1 = context$3$0.t0;
                        context$3$0.next = 11;
                        return _p('Title: (' + templateObj.title + ') ');

                    case 11:
                        context$3$0.t2 = context$3$0.sent;

                        if (context$3$0.t2) {
                            context$3$0.next = 14;
                            break;
                        }

                        context$3$0.t2 = templateObj.title;

                    case 14:
                        context$3$0.t3 = context$3$0.t2;
                        context$3$0.next = 17;
                        return _p('Subtitle: (' + templateObj.subtitle + ') ');

                    case 17:
                        context$3$0.t4 = context$3$0.sent;

                        if (context$3$0.t4) {
                            context$3$0.next = 20;
                            break;
                        }

                        context$3$0.t4 = templateObj.subtitle;

                    case 20:
                        context$3$0.t5 = context$3$0.t4;
                        context$3$0.next = 23;
                        return _p('Description: (' + templateObj.description + ') ');

                    case 23:
                        context$3$0.t6 = context$3$0.sent;

                        if (context$3$0.t6) {
                            context$3$0.next = 26;
                            break;
                        }

                        context$3$0.t6 = templateObj.description;

                    case 26:
                        context$3$0.t7 = context$3$0.t6;
                        context$3$0.next = 29;
                        return _p('Author: (' + templateObj.author + ') ');

                    case 29:
                        context$3$0.t8 = context$3$0.sent;

                        if (context$3$0.t8) {
                            context$3$0.next = 32;
                            break;
                        }

                        context$3$0.t8 = templateObj.author;

                    case 32:
                        context$3$0.t9 = context$3$0.t8;
                        context$3$0.next = 35;
                        return _p('Author bio: (' + templateObj.bio + ') ');

                    case 35:
                        context$3$0.t10 = context$3$0.sent;

                        if (context$3$0.t10) {
                            context$3$0.next = 38;
                            break;
                        }

                        context$3$0.t10 = templateObj.bio;

                    case 38:
                        context$3$0.t11 = context$3$0.t10;
                        context$3$0.next = 41;
                        return _p('Theme: (' + templateObj.theme + ') ');

                    case 41:
                        context$3$0.t12 = context$3$0.sent;

                        if (context$3$0.t12) {
                            context$3$0.next = 44;
                            break;
                        }

                        context$3$0.t12 = templateObj.theme;

                    case 44:
                        context$3$0.t13 = context$3$0.t12;
                        context$3$0.next = 47;
                        return _p('Preprocessor: (' + templateObj.preprocessor + ') ');

                    case 47:
                        context$3$0.t14 = context$3$0.sent;

                        if (context$3$0.t14) {
                            context$3$0.next = 50;
                            break;
                        }

                        context$3$0.t14 = templateObj.preprocessor;

                    case 50:
                        context$3$0.t15 = context$3$0.t14;
                        config = {
                            name: context$3$0.t1,
                            title: context$3$0.t3,
                            subtitle: context$3$0.t5,
                            description: context$3$0.t7,
                            author: context$3$0.t9,
                            bio: context$3$0.t11,
                            theme: context$3$0.t13,
                            preprocessor: context$3$0.t15
                        };

                        process.stdin.pause();

                        // create the configuration file
                        _fs2['default'].writeFile(manifest, JSON.stringify(_Object$assign({}, templateObj, config), null, 4), function (err) {
                            if (err) {
                                reject(err);
                                return;
                            }
                            fulfill();
                        });

                    case 54:
                    case 'end':
                        return context$3$0.stop();
                }
            }, callee$2$0, this);
        }))();
    });
}

function init(sitePath) {
    var skeletonPath, npm;
    return _regeneratorRuntime.async(function init$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                skeletonPath = _path2['default'].join(_config.rootdir, 'bin/skeleton');
                context$1$0.next = 3;
                return _regeneratorRuntime.awrap(mkdirpAsync(sitePath));

            case 3:
                context$1$0.next = 5;
                return _regeneratorRuntime.awrap(ncpAsync(skeletonPath, sitePath, { stopOnErr: true }));

            case 5:
                console.log(clc.message('Harmonic skeleton started at: ' + _path2['default'].resolve(sitePath)));

                context$1$0.next = 8;
                return _regeneratorRuntime.awrap(config(sitePath, true));

            case 8:

                console.log(clc.info('\nInstalling dependencies...'));
                context$1$0.next = 11;
                return _regeneratorRuntime.awrap(npmLoadAsync());

            case 11:
                npm = context$1$0.sent;
                context$1$0.prev = 12;
                context$1$0.next = 15;
                return _regeneratorRuntime.awrap((0, _bluebird.promisify)(npm.commands.install)(sitePath, []));

            case 15:
                context$1$0.next = 20;
                break;

            case 17:
                context$1$0.prev = 17;
                context$1$0.t0 = context$1$0['catch'](12);

                console.error((0, _dedent2['default'])(_taggedTemplateLiteral(['Command ', ' failed.\n             Make sure you are connected to the internet and execute the command above in your Harmonic skeleton directory.'], ['Command ', ' failed.\n             Make sure you are connected to the internet and execute the command above in your Harmonic skeleton directory.']), clc.error('npm install')));

            case 20:

                console.log('\n' + clc.info((0, _dedent2['default'])(_taggedTemplateLiteral(['Your Harmonic website skeleton was successfully created!\n         Now, browse the project directory and have fun.'], ['Your Harmonic website skeleton was successfully created!\n         Now, browse the project directory and have fun.']))));

            case 21:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[12, 17]]);
}

/**
 * @param {string} type - The new file's type. Can be either 'post' or 'page'.
 * @param {string} title - The new file's title.
 */
function newFile(passedPath, type, title, autoOpen) {
    var sitePath = (0, _helpers.findHarmonicRoot)(passedPath);

    if (!sitePath) {
        (0, _helpers.displayNonInitializedFolderErrorMessage)();
        throw new _helpers.MissingFileError();
    }

    var langs = (0, _helpers.getConfig)(sitePath).i18n.languages;
    // TODO use template literal + dedent
    var template = '<!--\n' + 'layout: ' + type + '\n' + 'title: ' + title + '\n' + 'date: ' + new Date().toJSON() + '\n' + 'comments: true\n' + 'published: true\n' + 'keywords:\n' + 'description:\n' + 'categories:\n' + '-->\n' + '# ' + title;
    var filedir = _path2['default'].join(sitePath, type === 'post' ? _config.postspath : _config.pagespath);
    var filename = (0, _helpers.titleToFilename)(title);

    langs.forEach(function (lang) {
        var fileLangDir = _path2['default'].join(filedir, lang);
        var fileW = _path2['default'].join(fileLangDir, filename);
        _mkdirp2['default'].sync(fileLangDir);
        _fs2['default'].writeFileSync(fileW, template);
        if (autoOpen) {
            openFile('text', sitePath, fileW);
        }
    });

    console.log(clc.info(type[0].toUpperCase() + type.slice(1) + ' "' + title + '" was successefuly created, check your /src/' + type + 's folder'));
}

function run(passedPath, port, autoOpen) {
    var sitePath, isBuilding, pendingBuild, bs, buildSite, doBuildSite, watcher;
    return _regeneratorRuntime.async(function run$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                doBuildSite = function doBuildSite() {
                    return _regeneratorRuntime.async(function doBuildSite$(context$2$0) {
                        while (1) switch (context$2$0.prev = context$2$0.next) {
                            case 0:
                                if (!isBuilding) {
                                    context$2$0.next = 3;
                                    break;
                                }

                                pendingBuild = true;
                                return context$2$0.abrupt('return');

                            case 3:

                                isBuilding = true;
                                context$2$0.next = 6;
                                return _regeneratorRuntime.awrap(buildSite());

                            case 6:
                                isBuilding = false;

                            case 7:
                            case 'end':
                                return context$2$0.stop();
                        }
                    }, null, this);
                };

                buildSite = function buildSite() {
                    return _regeneratorRuntime.async(function buildSite$(context$2$0) {
                        while (1) switch (context$2$0.prev = context$2$0.next) {
                            case 0:
                                context$2$0.prev = 0;
                                context$2$0.next = 3;
                                return _regeneratorRuntime.awrap((0, _core.build)(sitePath));

                            case 3:
                                if (!pendingBuild) {
                                    bs.reload();
                                }
                                context$2$0.next = 10;
                                break;

                            case 6:
                                context$2$0.prev = 6;
                                context$2$0.t0 = context$2$0['catch'](0);

                                console.error(clc.error('Build error:'));
                                console.error(context$2$0.t0.stack || context$2$0.t0.toString());

                            case 10:
                                if (!pendingBuild) {
                                    context$2$0.next = 14;
                                    break;
                                }

                                pendingBuild = false;
                                context$2$0.next = 14;
                                return _regeneratorRuntime.awrap(buildSite());

                            case 14:
                            case 'end':
                                return context$2$0.stop();
                        }
                    }, null, this, [[0, 6]]);
                };

                sitePath = (0, _helpers.findHarmonicRoot)(passedPath);

                if (sitePath) {
                    context$1$0.next = 6;
                    break;
                }

                (0, _helpers.displayNonInitializedFolderErrorMessage)();
                throw new _helpers.MissingFileError();

            case 6:
                isBuilding = false;
                pendingBuild = false;
                bs = _browserSync2['default'].create();
                context$1$0.next = 11;
                return _regeneratorRuntime.awrap((0, _bluebird.fromNode)(function (cb) {
                    bs.init({
                        server: {
                            baseDir: _path2['default'].join(sitePath, 'public'),
                            middleware: [function (req, res, next) {
                                if (isBuilding) {
                                    // TODO deliver browser-sync snippet when requesting a html file
                                    res.writeHead(503);
                                    res.end();
                                    return;
                                }
                                next();
                            }]
                        },
                        port: port,
                        open: autoOpen
                    }, cb);
                }));

            case 11:

                console.log(clc.info('Harmonic site is running.'));

                watcher = _chokidar2['default'].watch(['src', 'harmonic.json', _path2['default'].join('node_modules', (0, _helpers.getConfig)(sitePath).theme)], {
                    // interval: 1000,
                    // atomic: false,
                    ignoreInitial: true,
                    // ignorePermissionErrors: true,
                    cwd: sitePath
                }).on('add', doBuildSite)
                // .on('addDir', doBuildSite)
                .on('change', doBuildSite).on('unlink', doBuildSite)
                // .on('unlinkDir', doBuildSite)
                .on('error', function (error) {
                    return console.error('Error occurred', error);
                });
                context$1$0.next = 15;
                return _regeneratorRuntime.awrap((0, _bluebird.fromNode)(function (cb) {
                    return watcher.on('ready', cb);
                }));

            case 15:
                console.log(clc.info('Watching Harmonic project for changes...'));

                return context$1$0.abrupt('return', { watcher: watcher });

            case 17:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
}
/*eslint-disable camelcase*/

/*eslint-enable camelcase*/

// TODO watcher.unwatch and watcher.add when selected theme changes in harmonic.json