'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _fs = require('fs');

var _path = require('path');

var _cliColor2 = require('cli-color');

var _cliColor3 = _interopRequireDefault(_cliColor2);

exports.cliColor = cliColor;
exports.isHarmonicProject = isHarmonicProject;
exports.getConfig = getConfig;
exports.titleToFilename = titleToFilename;
exports.findHarmonicRoot = findHarmonicRoot;
exports.displayNonInitializedFolderErrorMessage = displayNonInitializedFolderErrorMessage;

// CLI color
function cliColor() {
    return {
        info: _cliColor3['default'].green,
        error: _cliColor3['default'].red,
        warn: _cliColor3['default'].yellowBright,
        message: _cliColor3['default'].yellow
    };
}

// Friendly message for non-initialized folder
function displayNonInitializedFolderErrorMessage() {
    var clc = cliColor();

    console.log(clc.warn('It seems this is not an Harmonic project yet. \n') + clc.warn('Check your directory or run ') + clc.info.bgWhite.italic(' harmonic init ') + clc.warn(' to start a new Harmonic project.'));
}

// Check if harmonic.json file exists
function isHarmonicProject(sitePath) {
    try {
        getConfig(sitePath);
        return true;
    } catch (e) {
        return false;
    }
}

// Find harmonic.json. Returns path or false.
function findHarmonicRoot(sitePath) {
    var currentPath = (0, _path.resolve)(sitePath);
    var oldPath = '';

    // Climb directories up until finding a `harmonic.json` file, if it is not found then return false.
    while (!isHarmonicProject(currentPath)) {
        oldPath = currentPath;
        currentPath = (0, _path.resolve)(currentPath, '..');

        if (oldPath === currentPath) {
            // reached root folder, return false;
            return false;
        }
    }

    return currentPath;
}

function getConfig(sitePath) {
    return JSON.parse((0, _fs.readFileSync)((0, _path.join)(sitePath, 'harmonic.json')).toString());
}

function titleToFilename(title) {
    return title.replace(/[^a-z0-9]+/gi, '-').replace(/^-+|-+$/g, '').toLowerCase() + '.md';
}

// Note: class declarations are not hoisted, so this can't be listed in the exports statement at the top of this file.

var MissingFileError = (function (_Error) {
    _inherits(MissingFileError, _Error);

    function MissingFileError() {
        var file = arguments.length <= 0 || arguments[0] === undefined ? 'harmonic.json' : arguments[0];

        _classCallCheck(this, MissingFileError);

        _get(Object.getPrototypeOf(MissingFileError.prototype), 'constructor', this).call(this);
        this.name = 'MissingFileError';
        this.file = file;
        this.message = 'Missing file: ' + this.file;
        delete this.stack;
    }

    return MissingFileError;
})(Error);

exports.MissingFileError = MissingFileError;